using System.Drawing;
using Infragistics.Documents.Excel;
using Infragistics.Win.UltraWinGrid.ExcelExport;
using com.airbus.pm00.model;
using com.airbus.pm00.controller.util;
using System.Reflection;
using Office;
using System.Windows.Forms;
using System.IO;
using System.Xml;

namespace com.airbus.pm00.controller
{
    public class CtrlExcelExportActions
    {
        private static CtrlExcelExportActions _CtrlExcelExportActions_Singleton = null;
        public int lastActionRow = 0;

        public CtrlExcelExportActions()
        {
        }
        
        public static CtrlExcelExportActions GetInstance()
        {
            //creer une nouvelle instance s il n'en existe pas deja une autre
            if (_CtrlExcelExportActions_Singleton == null) _CtrlExcelExportActions_Singleton = new CtrlExcelExportActions();
            return _CtrlExcelExportActions_Singleton;
        }

        public void CellExported(CellExportedEventArgs e)
        {
            if (e.GridColumn.Editor.ToString() == "Infragistics.Win.FormattedLinkLabel.FormattedLinkEditor")
            {
                //CellValue is the text with balises
                string CellValue = e.GridRow.Cells[e.GridColumn.Index].Value.ToString();
                //Replace "&edsp;" by "&#32;" in order to be recognized by xml document
                CellValue = CellValue.Replace("&edsp;", "&#32;");

                //Cell text is the text without balises
                string CellText = e.GridRow.Cells[e.GridColumn.Index].Text;
                //Delete \r . Only \n is needed for line return
                CellText = CellText.Replace("\r", "");

                if (CellValue.Contains("<span") && CellValue.Contains("</span>"))
                {
                    FormattedString StringValue = new FormattedString(CellText);
                    e.CurrentWorksheet.Rows[e.CurrentRowIndex].Cells[e.CurrentColumnIndex].Value = StringValue;
                    MemoryStream ms = new MemoryStream(System.Text.ASCIIEncoding.ASCII.GetBytes("<span>" + CellValue + "</span>"));
                    XmlDocument xmlDoc = new XmlDocument();
                    xmlDoc.Load(ms);
                    XmlNode Node = xmlDoc.SelectSingleNode("span");
                    CtrlUtils.AddSpanPositions(Node, 0);
                    CtrlUtils.FormatInfragisticsExcelString(StringValue, Node);
                }
            }
        }

        public void BeginExport(BeginExportEventArgs e)
        {
            lastActionRow = 0;
        }

        //Cancel the row exporting when the corresponding linked sheet is not exported
        public void RowExporting(RowExportingEventArgs e, int columnOffset)
        {
            switch (e.GridRow.Cells["C_ACTION_ORIG"].Value.ToString())
            {
                case "Milestone":
                    if (!CtrlMain.GetInstance().excelExportController.uiExcelExport.ClbListCheckbox.CheckedItems.Contains(CtrlExcelExport.CNameMilestoneWS))
                        e.Cancel = true;
                    else
                    {   //Correction point 157 21/09/2010)
                        Infragistics.Documents.Excel.Worksheet ws =  e.Workbook.Worksheets[CtrlExcelExport.CNameMilestoneWS];
                        bool exist = false; int columnIndex = -1;
                        foreach (WorksheetCell cell in ws.Rows[1].Cells) //Recherche de la colonne rfrence
                            if ((cell.Value != null) && (cell.Value.ToString() == "ID")) columnIndex = cell.ColumnIndex;
                        if (columnIndex != -1)
                        {
                            foreach (WorksheetRow row in ws.Rows)
                            {
                                if (row.Cells[columnIndex].Value != null)
                                {
                                    if (row.Cells[columnIndex].Value.ToString() == e.GridRow.Cells["C_ACTION_REF"].Value.ToString())
                                        exist = true;
                                }
                            }
                            if (!exist) e.Cancel = true;
                        }
                    }
                    break;
                case "RFC":
                    if (!CtrlMain.GetInstance().excelExportController.uiExcelExport.ClbListCheckbox.CheckedItems.Contains(CtrlExcelExport.CNameRFCWS))
                        e.Cancel = true;
                    else
                    {   //Correction point 157 21/09/2010)
                        Infragistics.Documents.Excel.Worksheet ws = e.Workbook.Worksheets[CtrlExcelExport.CNameRFCWS];
                        bool exist = false; int columnIndex = -1;
                        foreach (WorksheetCell cell in ws.Rows[0].Cells) //Recherche de la colonne rfrence
                            if ((cell.Value!=null) && (cell.Value.ToString() == "RFC")) columnIndex = cell.ColumnIndex;
                        if (columnIndex != -1)
                        {
                            foreach (WorksheetRow row in ws.Rows)
                            {
                                if (row.Cells[columnIndex].Value != null)
                                {
                                    if (row.Cells[columnIndex].Value.ToString() == e.GridRow.Cells["C_ACTION_REF"].Value.ToString())
                                        exist = true;
                                }
                            }
                            if (!exist) e.Cancel = true;
                        }
                    }
                    break;
                case "MSCN":
                    if (!CtrlMain.GetInstance().excelExportController.uiExcelExport.ClbListCheckbox.CheckedItems.Contains(CtrlExcelExport.CNameMscnWS))
                        e.Cancel = true;
                    else
                    {   //Correction point 157 21/09/2010)
                        Infragistics.Documents.Excel.Worksheet ws = e.Workbook.Worksheets[CtrlExcelExport.CNameMscnWS];
                        bool exist = false; int columnIndexFleet = -1; int columnIndexEpac = -1;
                        foreach (WorksheetCell cell in ws.Rows[0].Cells) //Recherche des colonnes rfrence
                            if ((cell.Value!=null) && (cell.Value.ToString() == "Fleet")) columnIndexFleet = cell.ColumnIndex;
                            else if ((cell.Value!=null) && (cell.Value.ToString() == "Epac/Tdu")) columnIndexEpac = cell.ColumnIndex;
                        if ((columnIndexFleet != -1) && (columnIndexEpac!= -1))
                        {
                            string fleet = e.GridRow.Cells["C_ACTION_REF"].Value.ToString().Substring(0,e.GridRow.Cells["C_ACTION_REF"].Value.ToString().IndexOf("-"));
                            string epac = e.GridRow.Cells["C_ACTION_REF"].Value.ToString().Substring(e.GridRow.Cells["C_ACTION_REF"].Value.ToString().IndexOf("-")+1);
                            foreach (WorksheetRow row in ws.Rows)
                            {
                                if ((row.Cells[columnIndexFleet].Value != null) && (row.Cells[columnIndexEpac].Value != null))
                                {
                                    if ((row.Cells[columnIndexFleet].Value.ToString() == fleet) && (row.Cells[columnIndexEpac].Value.ToString() == epac))
                                        exist = true;

                                }
                            }
                            if (!exist) e.Cancel = true;
                            
                        }
                    } 
                    break;
                case "Outstanding Point":
                    if (!CtrlMain.GetInstance().excelExportController.uiExcelExport.ClbListCheckbox.CheckedItems.Contains(CtrlExcelExport.CNameOutstandingPointWS))
                        e.Cancel = true;
                    else
                    {   //Correction point 157 21/09/2010)
                        Infragistics.Documents.Excel.Worksheet ws = e.Workbook.Worksheets[CtrlExcelExport.CNameOutstandingPointWS];
                        bool exist = false; int columnIndex = -1;
                        foreach (WorksheetCell cell in ws.Rows[0].Cells) //Recherche de la colonne rfrence
                            if ((cell.Value!=null) && (cell.Value.ToString() == "Title")) columnIndex = cell.ColumnIndex;
                        if (columnIndex != -1)
                        {
                            foreach (WorksheetRow row in ws.Rows)
                            {
                                if (row.Cells[columnIndex].Value != null)
                                {
                                    if (row.Cells[columnIndex].Value.ToString() == e.GridRow.Cells["C_ACTION_REF"].Value.ToString())
                                        exist = true;
                                }
                            }
                            if (!exist) e.Cancel = true;
                        }
                    } 
                    break;
            }          
        }

        public void EndExport(EndExportEventArgs e, int rowOffset, int columnOffset)
        {
            lastActionRow = e.CurrentRowIndex;
        }   
    }
}
